/*
 * dummy.c
 *
 *  Copyright (C)2014 : GreenSocs Ltd
 *      http://www.greensocs.com/ , email: info@greensocs.com
 *
 *  Developed by :
 *  Frederic Konrad   <fred.konrad@greensocs.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option)any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * This is a dummy clock.
 */

#include <linux/device.h>
#include <linux/clk.h>
#include <linux/clk-provider.h>
#include <linux/platform_device.h>
#include <linux/module.h>
#include <linux/of.h>

struct clk_dummy {
	struct clk_hw hw;
    unsigned long freq;
};

#define to_clk_dummy(_hw)	container_of(_hw, struct clk_dummy, hw)

static long dummy_round_rate(struct clk_hw *hw, unsigned long rate,
		unsigned long *parent_rate)
{
    struct clk_dummy *dummy = to_clk_dummy(hw);
	printk(KERN_INFO "round rate: %lu\n", rate);
	return rate;
}

static unsigned long dummy_recalc_rate(struct clk_hw *hw,
		unsigned long parent_rate)
{
    struct clk_dummy *dummy = to_clk_dummy(hw);
    printk(KERN_INFO "recalc rate: %lu\n", dummy->freq);
	return dummy->freq;
}

static int dummy_set_rate(struct clk_hw *hw, unsigned long rate,
		unsigned long parent_rate)
{
    struct clk_dummy *dummy = to_clk_dummy(hw);
    printk(KERN_INFO "set rate: %lu\n", rate);
    dummy->freq = rate;
	return 0;
}

static const struct clk_ops dummy_clk_ops = {
	.recalc_rate = dummy_recalc_rate,
	.round_rate = dummy_round_rate,
	.set_rate = dummy_set_rate,
};

static int dummy_clk_probe(struct platform_device *pdev)
{
	struct clk_dummy *data;
	struct clk_init_data init;
	struct clk *clk;
    u32 freq;

	data = devm_kzalloc(&pdev->dev, sizeof(*data), GFP_KERNEL);
	if (!data)
		return -ENOMEM;

	init.ops = &dummy_clk_ops;
	init.flags = CLK_IS_ROOT;
	init.num_parents = 0;
	data->hw.init = &init;

	if (of_property_read_string(pdev->dev.of_node, "clock-output-names",
			&init.name))
		init.name = pdev->dev.of_node->name;

	clk = devm_clk_register(&pdev->dev, &data->hw);
	if (IS_ERR(clk)) {
		dev_err(&pdev->dev, "clock registration failed\n");
		return PTR_ERR(clk);
	}
	of_clk_add_provider(pdev->dev.of_node, of_clk_src_simple_get, clk);

	/* Read the requested initial output frequency from device tree */
	if (!of_property_read_u32(pdev->dev.of_node, "clock-frequency",
                              &freq)) {
		clk_set_rate(clk, freq);
	}

	dev_info(&pdev->dev, "successfully probed... frequency: %u\n", freq);

	return 0;
}

static int dummy_clk_remove(struct platform_device *pdev)
{
	dev_info(&pdev->dev, "removed!\n");
	of_clk_del_provider(pdev->dev.of_node);
	return 0;
}

static struct of_device_id dummy_clk_match[] = {
	{ .compatible = "dummy-clk", },
	{},
};
MODULE_DEVICE_TABLE(of, dummy_clk_match);

static struct platform_driver dummy_clk_driver = {
	.probe		= dummy_clk_probe,
	.remove		= dummy_clk_remove,
	.driver		= {
		.name	= "dummy-clk",
		.owner	= THIS_MODULE,
		.of_match_table	= dummy_clk_match,
	},
};

static int __init dummy_clk_init(void)
{
    printk(KERN_INFO "dummy_clk registered!\n");
	return platform_driver_register(&dummy_clk_driver);
}

static void __exit dummy_clk_exit(void)
{
	platform_driver_unregister(&dummy_clk_driver);
    printk(KERN_INFO "dummy_clk unregistered!\n");
}

module_init(dummy_clk_init);
module_exit(dummy_clk_exit);

MODULE_AUTHOR("Frederic Konrad <fred.konrad@greensocs.com>");
MODULE_DESCRIPTION("Dummy clock.");
MODULE_LICENSE("GPL");
